.include "nes.h"
.include "ram.h"
.include "float.h"

.segment "BSS"

con_x: .res 1
con_y: .res 1
con_restofstring: .res 2

.segment "CODE"

;;
; Prints to the console.
.proc con_puts
  ldx con_x
  jsr vwfPuts
  stx con_x
  ; if it was a NUL, stop. Otherwise, print.
  cpy $00
  bne newline
  rts
  
newline:
  sta con_restofstring+1
  sty con_restofstring+0
  
  ; Begin next line
  jsr con_flush
  jsr clearLineImg
  ldx #0
  stx con_x
  inc con_y
  lda con_y
  cmp #24+4
  bcc :+
  lda #4
  sta con_y
:
  lda con_restofstring+1
  ldy con_restofstring+0
  jmp con_puts
.endproc

.proc con_flush
  lda con_y
  ldy #0
  cmp #16
  bcc notsecondplane
  ldy #8
  sbc #12
notsecondplane:
  jsr prep_copyLineImg
  lda nmis
:
  cmp nmis
  beq :-
  rts
.endproc

.proc con_init
  ldy #$04
  sty con_y
  ldy #$00
  sty last_PPUCTRL
  sty PPUMASK
  sty con_x
  lda #VBLANK_NMI
  sta PPUCTRL
  tya
  ldx #$20
  jsr ppu_clear_nt
  jsr clearLineImg
  lda #$23
  sta PPUADDR
  lda #$C8
  sta PPUADDR
  lda #$AA
  jsr twelveattrrows
  lda #$FF
  jsr twelveattrrows
  
  lda #$3F
  ldx #$00
  sta PPUADDR
  stx PPUADDR
palloop:
  lda con_palette,x
  sta PPUDATA
  inx
  cpx #16
  bcc palloop

  ; Fill nametable
  lda #$41
  jsr onerectrgn
  lda #$44
  jsr onerectrgn

  ; Clear pattern table
  lda #$00
  sta PPUADDR
  ldx #$00
  stx PPUADDR
  txa
  ldy #16
chrclrloop:
  sta PPUDATA
  dex
  bne chrclrloop
  dey
  bne chrclrloop

  lda #$00
  sta PPUADDR
  sta PPUADDR
  lda #$AA
  ldx #8
grayloop:
  sta PPUDATA
  eor #$FF
  dex
  bne grayloop

  lda #BG_ON
  sta last_PPUMASK
  lda #VBLANK_NMI|BG_0000|OBJ_1000
  sta last_PPUCTRL
  rts
onerectrgn:
  lsr a
  sta $03
  lda #$08<<1
  ror a
  sta $02
  lda #$40
  sta $04
  lda #16
  sta $05
  sta $06
  lda #12
  sta $07
  jmp draw_recttileregion
twelveattrrows:
  jsr fourattrrows
  jsr fourattrrows
fourattrrows:
  bit PPUDATA
  bit PPUDATA
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  bit PPUDATA
  bit PPUDATA
  rts
.endproc

.segment "RODATA"
LF = $0A
CURRENCY=$80
COPR = $81
con_palette:
  .byte $02,$1F,$1F,$1F, $02,$1F,$1F,$1F, $02,$38,$02,$38, $02,$02,$38,$38

.segment "BANK03"
.export con_demo_03
.proc con_demo_03
  jsr con_init
  lday #msg_hello
  jsr con_puts
  dfplda timemachine_cost
  lda #115
  ldy #<-1
  ldx #DFPREG_B
  jsr dfpfromu8
  ldx #DFPREG_B
  jsr dfpmul
  .if 1
    jsr dfpdump_a
  .else
    lday #ticker+Ticker::popup_text
    ldx #8
    jsr dfpbeautify_a
    lday #ticker+Ticker::popup_text
    jsr con_puts
  .endif

  jsr con_flush
loop:
  lda nmis
:
  cmp nmis
  beq :-
  jsr read_pads
  lda new_keys+0
  and #KEY_A|KEY_START
  beq loop
  rtl
.endproc

msg_hello:
  .byte "This ROM implements",LF
  .byte "SMBC Theater's MMO",LF
  .byte "Press A to view title screen",LF,LF
  .byte "Now what?",LF
  .byte "Calculate building costs",LF
  .byte "Draw building screen",LF
  .byte "Buy buildings",LF
  .byte "Playtime task scheduler",LF
  .byte "Calculate gains",LF
  .byte "Display cookies and CPS",LF
  .byte "  when dirty",LF
  .byte "Release ",34,"Classic",34,LF
  .byte "Save progress",LF
  .byte "Cursor strip",LF
  .byte "DFP floor divide, format",LF
  .byte "  payoff time, triangular root",LF
  .byte CURRENCY,0
msg_lfor:
  .byte LF,"...or ",0
msg_cmpresult:
  .byte LF,"cmpresult ",0
timemachine_cost:
  .byte  1,23,45,67,89,00,$84
flavored_cookie_cost:
  .byte  0,00,99,99,99,99,$85
fp_123whatever:
  .byte 12,34,56, 0, 0, 0,$82
fp_456_78:
  .byte  4,56,78, 0, 0, 0,$81

