.include "ram.h"
.include "mbyt.h"
.include "float.h"


.segment "BSS"
cookies: .res DFPPREC+1
cookiesEarned: .res DFPPREC+1
cookiesPerSecond: .res DFPPREC+1
cookiesPerClick: .res DFPPREC+1
cookiesDirty: .res 1


.segment "CODE"
;;
; To be called every second.
.proc earn_second
  dfpldb cookiesPerSecond
.endproc
;;
; Earns the number of cookies in DFPREG_B.
.proc earn_reg_b
  dfplda cookies
  ldx #DFPREG_B
  jsr dfpadd
  dfpsta cookies
  dfplda cookiesEarned
  ldx #DFPREG_B
  jsr dfpadd
  dfpsta cookiesEarned
.endproc
.proc set_cookie_count_dirty
  ; TO DO: set appropriate dirty flag for redrawing cookie counts,
  ; redrawing buildings, unlocking upgrades and achievements, etc.
  rts
.endproc

;;
; Earns the cookies per click.
.proc earn_click
  dfpldb cookiesPerClick
  jmp earn_reg_b
.endproc

;;
; Subtracts a purchase price in B from cookies in bank.
; @return A>=0: purchase succeeded; A<0: purchase failed
.proc spend_reg_b
  dfplda cookies
  ldx #DFPREG_B
  jsr dfpsub
  bmi no_purchase
  dfpsta cookies
  jsr set_cookie_count_dirty
  lda #0
no_purchase:
  rts
.endproc

;;
; Removes from both cookies and cookies earned, whether the player
; has that many or not.
; Used only for "Ruin" effect of red cookies
.proc dissolve_reg_b
  dfplda cookies
  ldx #DFPREG_B
  jsr dfpsub
  bpl :+
  dfplda #0
:
  dfpsta cookies
  dfplda cookiesEarned
  ldx #DFPREG_B
  jsr dfpsub
  bpl :+
  dfplda #0
:
  dfpsta cookiesEarned
  jmp set_cookie_count_dirty
.endproc

TALL_DIGITS_BASE = $16
topbarbuf_line1 = PB53_outbuf + $40
topbarbuf_line2 = topbarbuf_line1+9
topbarbuf_label = topbarbuf_line1+18

;;
; Renders 9-digit  count to a 9x2 tile map for header
; @param DFPREG_A count
; @param A which area to write to (0 for cookies, 1 for CpS)
;          affects destination address and prefix tile address
.proc build_topbarbuf
  pha
  ldx #9
  lday #topbarbuf_line1
  jsr dfpbeautify_a

gotpfx = 2
  ; now Y is length and X is what prefix (0=none, 1=k, 8=E)
  stx gotpfx
  ldx #8
  dey
  bmi lpadstart
shiftingloop:
  ; Transform nondigits to $00 tiles and 
  lda topbarbuf_line1,y
  cmp #'0'
  bcc isspace
  cmp #'9'+1
  bcs isspace
  adc #<(TALL_DIGITS_BASE - '0')
  bne have_tile
isspace:
  lda #0
have_tile:
  sta topbarbuf_line1,x
  beq :+
  eor #$30
:
  sta topbarbuf_line2,x
  dey
  bmi lpadstart
  dex
  bpl shiftingloop
  
  ; Now pad on the left with blank tiles
lpadstart:
  dex
  bmi no_lpad
  lda #0
lpadloop:
  sta topbarbuf_line1,x
  sta topbarbuf_line2,x
  dex
  bpl lpadloop
no_lpad:

  pla
  pha
  tax  ; X=0 for cookies or 1 for CpS
  lda gotpfx
  beq no_setup_sipfx
  ldy pfx_basetile,x
  sty topbarbuf_line1+7
  iny
  sty topbarbuf_line2+7
  iny
  sty topbarbuf_line1+8
  iny
  sty topbarbuf_line2+8

  ; TO DO: fill with appropriate tiles
  ; $38-$3B

  ldx gotpfx
  dex
  txa
  sec
  
  ; Now load the glyph of the SI prefix (0=k, 7=Y)
  ; to the first 64 bytes of the transfer buffer.
  ; A: item id (0-15 or so)
  ; X: offset (0 or 64)
  ldx #0
  ldy #<metric_pfx_imagedata
  sty ciSrc
  ldy #>metric_pfx_imagedata
  sty ciSrc+1
  asl a
  beq not_img0
  
  ; There's a table of segment offsets at the start of menu_furni,
  ; with each entry big-endian
  tay
  lda metric_pfx_imageptrs+1,y
  clc
  adc ciSrc
  sta ciSrc
  lda metric_pfx_imageptrs+0,y
  adc ciSrc+1
  sta ciSrc+1
not_img0:
  stx ciBufStart
  txa
  clc
  adc #64
  sta ciBufEnd
  jsr unpb53_some
no_setup_sipfx:
  ldy #4
  ldx #4
  pla
  pha
  beq labelloop
  ldy #9
labelloop:
  lda cookies_cps_labels,y
  sta topbarbuf_label,x
  dey
  dex
  bpl labelloop
  pla
  tax
  lda #$20
  jsr prep_copy_topbarbuf
  rts
.endproc

metric_pfx_imageptrs:
  .incbin "obj/nes/kavoon14_metric.pb53"
metric_pfx_imagedata = metric_pfx_imageptrs + 2 * 8
pfx_basetile: .byte $38, $3C
cookies_cps_labels:
  mbyt "08090A0B0C"
  mbyt "080E0F0000"
