;
; NES controller and mouse reading code
; Copyright 2009-2011 Damian Yerrick
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;

;
; 2011-07: Damian Yerrick added labels for the local variables and
;          copious comments and made USE_DAS a compile-time option
;

.export read_pads
.exportzp cur_keys, new_keys, joyportmasks

JOY1      = $4016
JOY2      = $4017

.segment "ZEROPAGE"
cur_keys:      .res 8
new_keys:      .res 2

; %01: Use NES/FCAV/clone plug-in controller or Famicom hardwired controller
; %10: Use Famicom DA15 plug-in controller
; %11: Use both, OR'd together
; In OR'd mode (%11), a hardwired controller will return $FF after
; its report, interfering with use of a plug-in mouse.  To work
; around this, perform mouse detection on both sides at reset time
; with both %01 and %10 settings.
joyportmasks:  .res 2

.segment "CODE"
.proc read_pads
thisRead = 0
lastFrameKeys = 4

  ; store the current keypress state to detect key-down later
  lda cur_keys
  sta lastFrameKeys
  lda cur_keys+1
  sta lastFrameKeys+1
  ldx #1  ; Strobe the mouse
  stx JOY1
  dex
  stx JOY1
  
  ; TO DO: I foresee problems with RF Famicoms. May have to use
  ; separate masks for slots 1 and 2.
byteloop:
  lda #$01        ; set up ring counter to terminate read loop
  sta thisRead+1  ; when the 1 gets shifted back out into C
bitloop:
  lda JOY1           ; read player 1's controller
  and joyportmasks+0 ; ignore D2-D7
  cmp #1             ; CLC if A=0, SEC if A>=1
  rol thisRead       ; put one bit in the register
  lda JOY2           ; read player 2's controller the same way
  and joyportmasks+1
  cmp #1
  rol thisRead+1
  bcc bitloop        ; once $01 has been shifted 8 times, we're done
  lda thisRead
  sta cur_keys,x
  inx
  lda thisRead+1
  sta cur_keys,x
  inx
  cpx #8
  bcc byteloop

  ; For each player, find newly pressed keys.
  ldx #1
find_new_keys:
  lda cur_keys+2,x  ; If it's a mouse...
  and #%00001111
  cmp #1
  bne not_mouse
  lda cur_keys+2,x  ; ...swap the left and right buttons...
  and #%11000000
  asl a
  bcc :+
  ora #%01000000
:
  ora cur_keys,x
  sta cur_keys,x
not_mouse:

  lda lastFrameKeys,x   ; A = keys that were down last frame
  eor #$FF              ; A = keys that were up last frame
  and cur_keys,x        ; A = keys down now and up last frame
  sta new_keys,x
  dex
  bpl find_new_keys
  rts
.endproc

.proc change_mouse_speed
  lda #1
  sta JOY1
  ldx JOY1
  lsr a
  sta JOY1
.endproc
