.include "nes.h"
.include "ram.h"

.segment "BSS"
; If oam_ready is nonzero, a display list is ready to be copied
; to OAM.
oam_ready: .res 1
last_PPUMASK: .res 1
last_PPUCTRL: .res 1  ; bit 7 clear: skip enhanced NMI handler

; A BGUP (background update procedure) copies prepared data
; from RAM to video memory.
.align 2  ; jump vectors on 6502 MUST NOT cross pages
bgup_proc: .res 2
bgup_args: .res 4

.segment "CODE"

; BGUP: START OF ENTRY POINTS ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.align 256
; All BGUPs MUST have entry points in the same page, such as through
; a JMP.  They MUST NOT read $8000-$BFFF; code and data that they use
; MUST be in the fixed bank or in RAM.  They MUST return by jumping
; to nmi_finish to do some final cleanup (NOT RTS).  And the BGUP at
; $xx00 MUST be the no-op.
bgup_null:              jmp nmi_finish
bgup_copy_topbarbuf:    jmp bgup_copy_topbarbuf_1

; ACTUAL BACKGROUND UPDATE PROCEDURES ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.assert <bgup_null = 0, error, "bgup_null is not aligned"
.assert *-bgup_null<256, error, "bgup_copyLineImg not in bgup entry point page"
.proc bgup_copyLineImg
  lda bgup_args+1
  ldy bgup_args
  jsr copyLineImg
  jmp nmi_finish
.endproc

.proc bgup_copy_topbarbuf_1
  ; Copy prefix icon
  lda #$03
  sta PPUADDR
  lda bgup_args+2
  sta PPUADDR
  clc
  lda #0
iconloop:
  tax
  .repeat 4,I
    ldy PB53_outbuf+I,x
    sty PPUDATA
  .endrepeat
  adc #4
  cmp #64
  bcc iconloop
  
  ; Copy top row of digits
  lda bgup_args+1
  sta PPUADDR
  lda bgup_args+0
  sta PPUADDR
  ldx #0
toprowloop:
  lda topbarbuf_line1,x
  sta PPUDATA
  inx
  cpx #9
  bcc toprowloop
  
  ; Copy bottom row of digits and label (Cookies, CPS)
  lda bgup_args+1
  sta PPUADDR
  lda bgup_args+0
  ora #$20
  sta PPUADDR
bottomrowloop:
  lda topbarbuf_line1,x
  sta PPUDATA
  inx
  cpx #18+5
  bcc bottomrowloop
  jmp nmi_finish
.endproc

; BGUP: THE NMI HANDLER ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.proc nmi
  bit last_PPUCTRL
  bpl skip
  pha
  
  ; If OAM is ready, copy it as early as possible in vblank.
  ; PAL NES has problems if OAM copy doesn't finish before line 261.
  lda oam_ready
  beq oam_not_ready
  lda #0
  sta OAMADDR
  sta oam_ready
  lda #>OAM
  sta OAM_DMA
oam_not_ready:
  bit PPUSTATUS
  txa
  pha
  tya
  pha
  jmp (bgup_proc)
finish:
  lda #<bgup_null
  sta bgup_proc
  sta PPUSCROLL
  sta PPUSCROLL
  lda last_PPUCTRL
  sta PPUCTRL
  lda last_PPUMASK
  sta PPUMASK
  pla
  tay
  pla
  tax
  pla
skip:
  inc nmis
  rti
.endproc

nmi_finish = nmi::finish

.proc init_bgup
  lda #<bgup_null
  sta bgup_proc
  sta last_PPUCTRL
  sta last_PPUMASK
  lda #>bgup_null
  sta bgup_proc+1
  rts
.endproc

; BACKGROUND UPDATE BUILDER ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; These procedures decide which to run.

.proc prep_copyLineImg
  sta bgup_args+1
  sty bgup_args
  lda #<bgup_copyLineImg
  sta bgup_proc
  rts
.endproc

;;
; A: nametable ID ($20 or $24)
; X: 0 for copying to Cookies, 1 for copying to CPS
.proc prep_copy_topbarbuf
  sta bgup_args+1
  lda cookiecount_dstlo,x
  sta bgup_args+0
  lda cookiecount_pfxbaselo,x
  sta bgup_args+2
  lda #<bgup_copy_topbarbuf
  sta bgup_proc
  rts
.pushseg
.segment "RODATA"
cookiecount_pfxbaselo:
  .byte $80, $C0  ; prefix images stored in tiles $38 and $3C
cookiecount_dstlo:
  .byte $44, $52
.popseg
.endproc

